#ifndef _PALMOSGUI_H
#define _PALMOSGUI_H

#include "PalmOSGUI_dbfile.h"

// 4-chars-as-UInt32
#define makeType(a,b,c,d) ((((UInt32)a)<<24) | (((UInt32)b)<<16) | (((UInt32)c)<<8) | ((UInt32)d))

// Resource signatures
#define appSignature	((UInt32)	makeType('F','r','o','d'))
#define armletType	((DmResType)	makeType('A','r','m','l'))
#define romType		((DmResType)	makeType('R','O','M','z'))
#define tblType		((DmResType)	makeType('T','a','b','l'))
#define strType		((DmResType)	makeType('S','t','r','g'))
#define prefsType	((DmResType)	makeType('P','r','e','f'))
#define bitmapType	((DmResType)	makeType('T','b','m','p'))
#define dataType	((DmResType)	makeType('D','A','T','A'))
#define mainType	((DmResType)	makeType('M','A','I','N'))
#define crashType	((DmResType)	makeType('C','R','S','H'))
#define dbDataType	((UInt32)	makeType('d','a','t','a'))

// Resource indexes. Why 32? Well, i like some space...
// It is CRITICAL that these indexes agree with precalc.c and the Makefile!

// ROMz
#define IDX_KERNAL	32
#define IDX_BASIC	33
#define IDX_CHAR	34
#define IDX_1541	35

// Tabl
#define IDX_EXPTABLE		32
#define IDX_MULTIEXPTABLE	33
#define IDX_TEXTCOLORTABLE	34
#define IDX_TRITABLE		35
#define IDX_TRISAWTABLE		36
#define IDX_TRIRECTTABLE	37
#define IDX_SAWRECTTABLE	38
#define IDX_TRISAWRECTTABLE	39
#define IDX_EGTABLE		40
#define IDX_EGDRSHIFT		41
#define IDX_SAMPLETAB		42
#define IDX_SINTAB		43
#define IDX_RESONANCELP		44
#define IDX_RESONANCEHP		45

// Strg
#define IDX_ERRORS1541		32

// Arml
#define IDX_ARMLET		32

// Pref
#define IDX_PREFS		32

// CRSH
#define IDX_CRASH		32

// Intel XScale PXA CPU's

#define sonyHwrOEMDeviceIDCordoba	'crdb'
#define sonyHwrOEMDeviceIDGranada	'grnd'
#define sonyHwrOEMDeviceIDVerona	'vrna'
#define sonyHwrOEMDeviceIDMadrid	'mdrd'
#define sonyHwrOEMDeviceIDToledo	'tldo'

/* PEG-NX70(V) & NX60 */
#define sonyHwrOEMDeviceID_NX70		sonyHwrOEMDeviceIDCordoba
/* PEG-NZ90 */
#define sonyHwrOEMDeviceID_NZ90		sonyHwrOEMDeviceIDGranada
/* PEG-TG50 */
#define sonyHwrOEMDeviceID_TG50		sonyHwrOEMDeviceIDVerona
/* PEG-NX73 */
#define sonyHwrOEMDeviceID_NX73		sonyHwrOEMDeviceIDToledo
/* PEG-NX80 */
#define sonyHwrOEMDeviceID_NX80		sonyHwrOEMDeviceIDMadrid
/* Tungsten|C*/
#define palmDeviceID_TC			'MT64'
/* Tungsten|T3*/
#define palmDeviceID_T3			'Arz1'

// Texas Instruments OMAP CPU's
#define palmDeviceID_TT			'Frg1'
#define palmDeviceID_T2			'Frg2'
#define palmDeviceID_TE			'CCt1'
#define palmDeviceID_Zire21		'Zi21' 
#define palmDeviceID_Zire71		'Zpth'
#define palmDeviceID_Treo600		'H101'

// Motorola MXL CPU's
#define sonyHwrOEMDeviceIDGoku		'goku'
#define sonyHwrOEMDeviceIDGohan		'goha'
/* PEG-TJ35 */
#define sonyHwrOEMDeviceID_TJ35		sonyHwrOEMDeviceIDGoku
/* PEG-TJ25 */
#define sonyHwrOEMDeviceID_TJ25		sonyHwrOEMDeviceIDGohan
#define palmDeviceID_iQ3600		'3600'

// 0x80d0 0x80e0
// 0x8c54 0x8415
#define SNAPSHOT_SIZE_1 0x9000L
#define SNAPSHOT_SIZE_2 0x9000L

typedef UInt32 uint32;
typedef Int32 int32;
typedef UInt16 uint16;
typedef Int16 int16;
typedef UInt8 uint8;
typedef Int8 int8;

// Declarations
UInt32 Launch();
UInt32 CheckPrerequisites();
UInt32 CreateArmlet();
UInt32 CreateArmletData();
UInt32 InitGraphicsSound();
UInt32 RunArmlet();
MemPtr NewSwapped(UInt32);
void FreeSwapped(MemPtr);
MemPtr GetRes(DmResType, UInt32, char *);
void UngetRes(MemPtr);
MemPtr CopyRes(DmResType, UInt32, char *);
MemPtr CopyRes2(DmResType, UInt32, char *, int);
void UncopyRes(MemPtr);

Err Scan(DbFile *, UInt32, char *);
Err ScanIndex(FileRef, UInt32, char *);

// Motorola MXL CPU's
Boolean DeviceHasMXLCPU();
// Texas Instruments OMAP CPU
Boolean DeviceHasOMAPCPU();
// Intel XScale PXA CPU
Boolean DeviceHasPXACPU();

void Load(DbFile* dbf, UInt16 idx);
void SaveSnapshot(char *name, Int32 idx);

static Boolean AppHandleEvent(EventPtr e);
static Boolean HandleEventMain(EventPtr e);
static Boolean HandleEventGfxSound(EventPtr e);
static Boolean HandleEventJoystick(EventPtr e);
static Boolean HandleEventObscure(EventPtr e);
static Boolean HandleEventSampleKey(EventPtr e);

void InitGfxSoundDialog(FormType *f);
void ExitGfxSoundDialog(FormType *f, UInt16 choice);
void InitObscureDialog(FormType *f);
void ExitObscureDialog(FormType *f, UInt16 choice);
void InitJoystickDialog(FormType *f);

Err SoundCallback(void *, SndStreamRef, void *, UInt32);

// Arm->68k callbacks
UInt32 VBlank68k(void *);
Int32  ShowRequester68k(Int32 *);
void  *Malloc68k(UInt32);
UInt32 Free68k(void *);

#endif
