#include "palmposix.h"

#include "Display.h"

// No fopen. Files appear magically, delivered by the 68k

UInt32 fread(void *buf, UInt32 size, UInt32 n, FILE *f)
{
	int i, retval=0, rd, chunksz, ok;
	uint8 *b=(uint8 *) buf;

	for (i=0; i<n; i++) {
		// Read past end
		if (f->offset + size > f->size)
			return retval;
		// Chunk boundrary (size may be many times partSize)
		if (f->partOffset + size > f->partSize) {
			rd=0;
			while (rd < size) {
				chunksz=f->partSize-f->partOffset;
				if (chunksz > size-rd)
					chunksz=size-rd;
				ok=fread(b, chunksz, 1, f);
				if (!ok) return retval;
				rd+=chunksz;
				b+=chunksz;
			}
			retval++;
			continue;
		}
		// All chunk crossings taken care of above
		memcpy(b, f->part[f->currentPart].p+f->partOffset, size);
		retval++;
		b += size;
		f->offset += size;
		f->partOffset += size;
		if (f->partOffset == f->partSize) {
			f->partOffset=0;
			f->currentPart++;
		}
	}
	return retval;
}
UInt32 fwrite(const void *buf, UInt32 size, UInt32 n, FILE *f)
{
	int i, retval=0, wr, chunksz, ok;
	const uint8 *b=(const uint8 *) buf;

	for (i=0; i<n; i++) {
		// Write past end, not allowed
		if (f->offset + size > f->size)
			return retval;
		// Chunk boundrary (size may be many times partSize)
		if (f->partOffset + size > f->partSize) {
			wr=0;
			while (wr < size) {
				chunksz=f->partSize;
				if (chunksz > size-wr) chunksz=size-wr;
				ok=fwrite(b, chunksz, 1, f);
				if (!ok) return retval;
				wr+=chunksz;
				b+=chunksz;
			}
			retval++;
			continue;
		}
		// All chunk crossings taken care of above
		memcpy(f->part[f->currentPart].p+f->partOffset, b, size);
		retval += size;
		b += size;
		f->offset += size;
		f->partOffset += size;
		if (f->partOffset == f->partSize) {
			f->partOffset=0;
			f->currentPart++;
		}
	}
	return retval;
}
int16 fgetc(FILE *f)
{
	int16 retval;

	if (f->offset >= f->size)
		return EOF;

	retval=f->part[f->currentPart].p[f->partOffset]&0xff;
	f->partOffset++;
	f->offset++;
	if (f->partOffset == f->partSize) {
		f->partOffset=0;
		f->currentPart++;
	}
	return retval;
}
Int16 fputc(UInt16 c, FILE *f)
{
	Int16 retval;

	// Files can't grow
	if (f->offset >= f->size)
		return EOF;

	f->part[f->currentPart].p[f->partOffset]=(uint8) c;
	f->partOffset++;
	f->offset++;
	if (f->partOffset == f->partSize) {
		f->partOffset=0;
		f->currentPart++;
	}
	return retval;
}
UInt32 feof(FILE *f)
{
	return f->offset >= f->size;
}

void fflush(FILE *f )
{
}

Int32 fseek(FILE *f, Int32 n, UInt32 mode)
{
	switch (mode) {
		case SEEK_CUR: return fseek(f, f->offset+n, SEEK_SET);
		case SEEK_END: return fseek(f, f->size+n, SEEK_SET);
		case SEEK_SET: break;
		default: return -1;
	}
	// SEEK_SET it is.
	if (n>f->size || n<0) return -1;

	f->offset=n;
	f->currentPart=0;
	for (; n>=f->partSize; n-=f->partSize) {
		f->currentPart++;
	}
	f->partOffset=n;
}
UInt32 ftell(FILE *f)
{
	return f->offset;
}
void fclose(FILE *f){}

// Memory and string functions
void *memcpy(void *dst, const void *src, uint32 n)
{
	uint32 i;

	if (!src || !dst) return dst;
	for (i=0; i<n; i++)
		((char *)dst)[i]=((char *)src)[i];

	return dst;
}
void *memset(void *dst, int8 val, uint32 n)
{
	uint32 i;
	char *p=(char *)dst;

	if (!dst) return dst;
	for (i=0; i<n; i++)
		*p++ = val;

	return dst;
}
char *strchr(char *src, const char c)
{
	char *retval;

	if (!src) return NULL;
	for (retval=src; *retval; retval++)
		if (*retval==c)
			return retval;

	return NULL;
}
char *strncpy(char *dst, const char *src, uint32 n)
{
	uint32 i;

	if (!src || !dst) return dst;
	for (i=0; i<n; i++) {
		dst[i]=src[i];
		if (*dst=='\0')
			break;
	}
	return dst;
}
char *strcpy(char *dst, const char *src)
{
	if (!src || !dst) return dst;
	while (*src) *dst=*src;
	*dst='\0';

	return dst;
}
// asda < asdb: -1
// asd < asda: -1
int16 strcmp(const char *a, const char *b)
{
	if (!a && !b) return 0;
	if (!a) return -1;
	if (!b) return 1;

	for (; *a && *b; a++, b++) 
		if (*a!=*b) return *a-*b;

	if (*a) return 1;
	if (*b) return -1;
	return 0;
}
int16 memcmp(const uint8 *a, const uint8 *b, uint32 n)
{
	int i;

	if (!a && !b) return 0;
	if (!a) return -1;
	if (!b) return 1;

	for (i=0; i<n; i++)
		if (a[i]!=b[i]) return a[i]-b[i];

	return 0;
}
uint32 strlen(const char *src)
{
	uint32 i;

	if (!src) return 0;

	for (i=0; src[i]; i++);

	return i;
}
int32 atoi(const char *src)
{
	int8 neg=0, first=1;
	int32 retval=0;

	if (!src) return 0;

        for (; *src; src++) {
                if (*src=='-') {
                        if (!first) return retval;
                        neg=1;
		} else if (*src<'0' || *src>'9')
                        return retval;
		else 
                	retval=10*retval+*src-'0';
		first=0;
        }
	if (neg) return -retval;
	return retval;
}
char *strcat(char *dst, const char *src)
{
	if (!src || !dst) return dst;

	while (*dst) dst++;
	while (*src)
		*dst++=*src++;

	return dst;
}

char *hexfmt(uint32 d, uint16 len, char *dst)
{
	uint16 dlen, i,j;
	uint8 dgt;

	if (!dst) return dst;

	if (d>=0x10000000)
		dlen=8;
	else if (d>=0x1000000)
		dlen=7;
	else if (d>=0x100000)
		dlen=6;
	else if (d>=0x10000)
		dlen=5;
	else if (d>=0x1000)
		dlen=4;
	else if (d>=0x100)
		dlen=3;
	else if (d>=0x10)
		dlen=2;
	else if (d>=0x1)
		dlen=1;
	else 
		dlen=0;

	if (dlen>len) len=dlen;

	for (j=len-1; j>=i; j--) {
		dgt=d&0xf;
		if (dgt<10)
			dst[j]='0'+dgt;
		else
			dst[i]='a'+dgt-10;
		d=d>>4;
	}
	dst[len]='\0';
	return dst;
}

/*
int rand(void);
float sin(float);
*/

#define kPceNativeWantA0                   (0x10000000)

void *operator new(unsigned long size)
{
	ArmletData *ad=(ArmletData *)GetR6();
	void *retval;

	PutR6(ad->r6org);
	retval=(void *)ad->call68k(ad->emulStateP, (uint32)ad->malloc, &size, 4 | kPceNativeWantA0);
	ad->r6org=GetR6();
	PutR6((uint32)ad);
	//ShowRequester(REQ_DEBUG, 0xa110c, size, (int32)retval, 0);
	return retval;
}
void operator delete(void *p)
{
	ArmletData *ad=(ArmletData *)GetR6();

	//ShowRequester(REQ_DEBUG, 0xf3ee, (int32)p, 0, 0);
	PutR6(ad->r6org);
	ad->call68k(ad->emulStateP, (uint32)ad->free, &p, 4);
	ad->r6org=GetR6();
	PutR6((uint32)ad);
}
void *operator new[](unsigned long size) { return ::operator new(size); }
void operator delete[](void *p) { ::operator delete(p); }



/*
uint32 GetR6()
{
	uint32 r6;
	__asm__("mov %0, r6" : "=r" (r6));

	return r6;
}
void PutR6(uint32 r6)
{
	uint32 r=r6;
	__asm__("mov r6, %0" : "=r" (r));
}
*/

